@@ -2,17 +2,19 @@
 .. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
 """
 
+import ntpath
 import platform
 import re
 import string
-from pathlib import Path
-from typing import Any, List, Optional
+import sys
+from pathlib import PurePath
+from typing import Any, Final, Optional
 
 from ._const import Platform
 from ._types import PathType, PlatformType
 
 
-_re_whitespaces = re.compile(r"^[\s]+$")
+_re_whitespaces: Final = re.compile(r"^[\s]+$")
 
 
 def validate_pathtype(
@@ -20,7 +22,7 @@ def validate_pathtype(
 ) -> None:
     from .error import ErrorReason, ValidationError
 
-    if _is_not_null_string(text) or isinstance(text, Path):
+    if _is_not_null_string(text) or isinstance(text, PurePath):
         return
 
     if allow_whitespaces and _re_whitespaces.search(str(text)):
@@ -33,12 +35,25 @@ def validate_pathtype(
 
 
 def to_str(name: PathType) -> str:
-    if isinstance(name, Path):
+    if isinstance(name, PurePath):
         return str(name)
 
     return name
 
 
+def is_nt_abspath(value: str) -> bool:
+    ver_info = sys.version_info[:2]
+    if ver_info <= (3, 10):
+        if value.startswith("\\\\"):
+            return True
+    elif ver_info >= (3, 13):
+        return ntpath.isabs(value)
+
+    drive, _tail = ntpath.splitdrive(value)
+
+    return ntpath.isabs(value) and len(drive) > 0
+
+
 def is_null_string(value: Any) -> bool:
     if value is None:
         return True
@@ -56,15 +71,15 @@ def _is_not_null_string(value: Any) -> bool:
         return False
 
 
-def _get_unprintable_ascii_chars() -> List[str]:
+def _get_unprintable_ascii_chars() -> list[str]:
     return [chr(c) for c in range(128) if chr(c) not in string.printable]
 
 
-unprintable_ascii_chars = tuple(_get_unprintable_ascii_chars())
+unprintable_ascii_chars: Final = tuple(_get_unprintable_ascii_chars())
 
 
-def _get_ascii_symbols() -> List[str]:
-    symbol_list: List[str] = []
+def _get_ascii_symbols() -> list[str]:
+    symbol_list: list[str] = []
 
     for i in range(128):
         c = chr(i)
@@ -77,12 +92,12 @@ def _get_ascii_symbols() -> List[str]:
     return symbol_list
 
 
-ascii_symbols = tuple(_get_ascii_symbols())
+ascii_symbols: Final = tuple(_get_ascii_symbols())
 
-__RE_UNPRINTABLE_CHARS = re.compile(
+__RE_UNPRINTABLE_CHARS: Final = re.compile(
     "[{}]".format(re.escape("".join(unprintable_ascii_chars))), re.UNICODE
 )
-__RE_ANSI_ESCAPE = re.compile(
+__RE_ANSI_ESCAPE: Final = re.compile(
     r"(?:\x1B[@-Z\\-_]|[\x80-\x9A\x9C-\x9F]|(?:\x1B\[|\x9B)[0-?]*[ -/]*[@-~])"
 )
 
@@ -116,25 +131,33 @@ def normalize_platform(name: Optional[PlatformType]) -> Platform:
     if not name:
         return Platform.UNIVERSAL
 
-    name = name.strip().casefold()
+    platform_str = name.strip().casefold()
 
-    if name == "posix":
+    if platform_str == "posix":
         return Platform.POSIX
 
-    if name == "auto":
-        name = platform.system().casefold()
+    if platform_str == "auto":
+        platform_str = platform.system().casefold()
 
-    if name in ["linux"]:
+    if platform_str in ["linux"]:
         return Platform.LINUX
 
-    if name and name.startswith("win"):
+    if platform_str and platform_str.startswith("win"):
         return Platform.WINDOWS
 
-    if name in ["mac", "macos", "darwin"]:
+    if platform_str in ["mac", "macos", "darwin"]:
         return Platform.MACOS
 
     return Platform.UNIVERSAL
 
 
-def findall_to_str(match: List[Any]) -> str:
-    return ", ".join([repr(text) for text in match])
+def findall_to_str(match: list[Any]) -> str:
+    uniq_list = set([repr(text) for text in match])
+    return ", ".join(uniq_list)
+
+
+def truncate_str(text: str, encoding: str, max_bytes: int) -> str:
+    str_bytes = text.encode(encoding)
+    str_bytes = str_bytes[:max_bytes]
+    # last char might be malformed, ignore it
+    return str_bytes.decode(encoding, "ignore")
