@@ -3,23 +3,27 @@
 """
 
 import itertools
-import ntpath
 import posixpath
 import re
-from pathlib import Path
-from typing import Optional, Pattern, Tuple
+import warnings
+from collections.abc import Sequence
+from pathlib import Path, PurePath
+from re import Pattern
+from typing import Final, Optional
 
 from ._base import AbstractSanitizer, AbstractValidator, BaseFile, BaseValidator
-from ._common import findall_to_str, to_str, validate_pathtype
+from ._common import findall_to_str, is_nt_abspath, to_str, truncate_str, validate_pathtype
 from ._const import DEFAULT_MIN_LEN, INVALID_CHAR_ERR_MSG_TMPL, Platform
 from ._types import PathType, PlatformType
-from .error import ErrorReason, InvalidCharError, ValidationError
-from .handler import NullValueHandler
+from .error import ErrorAttrKey, ErrorReason, InvalidCharError, ValidationError
+from .handler import ReservedNameHandler, ValidationErrorHandler
 
 
-_DEFAULT_MAX_FILENAME_LEN = 255
-_RE_INVALID_FILENAME = re.compile(f"[{re.escape(BaseFile._INVALID_FILENAME_CHARS):s}]", re.UNICODE)
-_RE_INVALID_WIN_FILENAME = re.compile(
+_DEFAULT_MAX_FILENAME_LEN: Final = 255
+_RE_INVALID_FILENAME: Final = re.compile(
+    f"[{re.escape(BaseFile._INVALID_FILENAME_CHARS):s}]", re.UNICODE
+)
+_RE_INVALID_WIN_FILENAME: Final = re.compile(
     f"[{re.escape(BaseFile._INVALID_WIN_FILENAME_CHARS):s}]", re.UNICODE
 )
 
@@ -30,8 +34,9 @@ class FileNameSanitizer(AbstractSanitizer):
         max_len: int = _DEFAULT_MAX_FILENAME_LEN,
         fs_encoding: Optional[str] = None,
         platform: Optional[PlatformType] = None,
-        check_reserved: bool = True,
-        null_value_handler: Optional[NullValueHandler] = None,
+        null_value_handler: Optional[ValidationErrorHandler] = None,
+        reserved_name_handler: Optional[ValidationErrorHandler] = None,
+        additional_reserved_names: Optional[Sequence[str]] = None,
         validate_after_sanitize: bool = False,
         validator: Optional[AbstractValidator] = None,
     ) -> None:
@@ -42,16 +47,17 @@ class FileNameSanitizer(AbstractSanitizer):
                 min_len=DEFAULT_MIN_LEN,
                 max_len=max_len,
                 fs_encoding=fs_encoding,
-                check_reserved=check_reserved,
+                check_reserved=True,
+                additional_reserved_names=additional_reserved_names,
                 platform=platform,
             )
 
         super().__init__(
             max_len=max_len,
             fs_encoding=fs_encoding,
-            check_reserved=check_reserved,
             null_value_handler=null_value_handler,
-            platform_max_len=_DEFAULT_MAX_FILENAME_LEN,
+            reserved_name_handler=reserved_name_handler,
+            additional_reserved_names=additional_reserved_names,
             platform=platform,
             validate_after_sanitize=validate_after_sanitize,
             validator=fname_validator,
@@ -64,22 +70,24 @@ class FileNameSanitizer(AbstractSanitizer):
             validate_pathtype(value, allow_whitespaces=not self._is_windows(include_universal=True))
         except ValidationError as e:
             if e.reason == ErrorReason.NULL_NAME:
-                if isinstance(value, Path):
+                if isinstance(value, PurePath):
                     raise
 
-                return self._null_value_handler(e)
+                return self._null_value_handler(e)  # type: ignore
             raise
 
         sanitized_filename = self._sanitize_regexp.sub(replacement_text, str(value))
-        sanitized_filename = sanitized_filename[: self.max_len]
+        sanitized_filename = truncate_str(sanitized_filename, self._fs_encoding, self.max_len)
 
         try:
             self._validator.validate(sanitized_filename)
         except ValidationError as e:
-            if e.reason == ErrorReason.RESERVED_NAME and e.reusable_name is False:
-                sanitized_filename = re.sub(
-                    re.escape(e.reserved_name), f"{e.reserved_name}_", sanitized_filename
-                )
+            if e.reason == ErrorReason.RESERVED_NAME:
+                replacement_word = self._reserved_name_handler(e)
+                if e.reserved_name != replacement_word:
+                    sanitized_filename = re.sub(
+                        re.escape(e.reserved_name), replacement_word, sanitized_filename
+                    )
             elif e.reason == ErrorReason.INVALID_CHARACTER and self._is_windows(
                 include_universal=True
             ):
@@ -103,10 +111,10 @@ class FileNameSanitizer(AbstractSanitizer):
                     platform=self.platform,
                 )
 
-        if isinstance(value, Path):
-            return Path(sanitized_filename)
+        if isinstance(value, PurePath):
+            return Path(sanitized_filename)  # type: ignore
 
-        return sanitized_filename
+        return sanitized_filename  # type: ignore
 
     def _get_sanitize_regexp(self) -> Pattern[str]:
         if self._is_windows(include_universal=True):
@@ -116,29 +124,37 @@ class FileNameSanitizer(AbstractSanitizer):
 
 
 class FileNameValidator(BaseValidator):
-    _WINDOWS_RESERVED_FILE_NAMES = ("CON", "PRN", "AUX", "CLOCK$", "NUL") + tuple(
-        f"{name:s}{num:d}" for name, num in itertools.product(("COM", "LPT"), range(1, 10))
+    _WINDOWS_RESERVED_FILE_NAMES: Final = (
+        ("CON", "PRN", "AUX", "CLOCK$", "NUL")
+        + tuple(f"{name:s}{num:d}" for name, num in itertools.product(("COM", "LPT"), range(0, 10)))
+        + tuple(
+            f"{name:s}{ssd:s}"
+            for name, ssd in itertools.product(
+                ("COM", "LPT"),
+                ("\N{SUPERSCRIPT ONE}", "\N{SUPERSCRIPT TWO}", "\N{SUPERSCRIPT THREE}"),
+            )
+        )
     )
-    _MACOS_RESERVED_FILE_NAMES = (":",)
+    _MACOS_RESERVED_FILE_NAMES: Final = (":",)
 
     @property
-    def reserved_keywords(self) -> Tuple[str, ...]:
+    def reserved_keywords(self) -> tuple[str, ...]:
         common_keywords = super().reserved_keywords
 
         if self._is_universal():
-            return (
+            word_set = set(
                 common_keywords
                 + self._WINDOWS_RESERVED_FILE_NAMES
                 + self._MACOS_RESERVED_FILE_NAMES
             )
+        elif self._is_windows():
+            word_set = set(common_keywords + self._WINDOWS_RESERVED_FILE_NAMES)
+        elif self._is_posix() or self._is_macos():
+            word_set = set(common_keywords + self._MACOS_RESERVED_FILE_NAMES)
+        else:
+            word_set = set(common_keywords)
 
-        if self._is_windows():
-            return common_keywords + self._WINDOWS_RESERVED_FILE_NAMES
-
-        if self._is_posix() or self._is_macos():
-            return common_keywords + self._MACOS_RESERVED_FILE_NAMES
-
-        return common_keywords
+        return tuple(sorted(word_set))
 
     def __init__(
         self,
@@ -147,13 +163,14 @@ class FileNameValidator(BaseValidator):
         fs_encoding: Optional[str] = None,
         platform: Optional[PlatformType] = None,
         check_reserved: bool = True,
+        additional_reserved_names: Optional[Sequence[str]] = None,
     ) -> None:
         super().__init__(
             min_len=min_len,
             max_len=max_len,
             fs_encoding=fs_encoding,
             check_reserved=check_reserved,
-            platform_max_len=_DEFAULT_MAX_FILENAME_LEN,
+            additional_reserved_names=additional_reserved_names,
             platform=platform,
         )
 
@@ -166,9 +183,11 @@ class FileNameValidator(BaseValidator):
         self.validate_abspath(unicode_filename)
 
         err_kwargs = {
-            "reason": ErrorReason.INVALID_LENGTH,
-            "platform": self.platform,
-            "fs_encoding": self._fs_encoding,
+            ErrorAttrKey.REASON: ErrorReason.INVALID_LENGTH,
+            ErrorAttrKey.PLATFORM: self.platform,
+            ErrorAttrKey.FS_ENCODING: self._fs_encoding,
+            ErrorAttrKey.BYTE_COUNT: byte_ct,
+            ErrorAttrKey.VALUE: unicode_filename,
         }
         if byte_ct > self.max_len:
             raise ValidationError(
@@ -186,11 +205,10 @@ class FileNameValidator(BaseValidator):
             )
 
         self._validate_reserved_keywords(unicode_filename)
+        self.__validate_universal_filename(unicode_filename)
 
         if self._is_windows(include_universal=True):
             self.__validate_win_filename(unicode_filename)
-        else:
-            self.__validate_unix_filename(unicode_filename)
 
     def validate_abspath(self, value: str) -> None:
         err = ValidationError(
@@ -200,19 +218,20 @@ class FileNameValidator(BaseValidator):
         )
 
         if self._is_windows(include_universal=True):
-            if ntpath.isabs(value):
+            if is_nt_abspath(value):
                 raise err
 
         if posixpath.isabs(value):
             raise err
 
-    def __validate_unix_filename(self, unicode_filename: str) -> None:
+    def __validate_universal_filename(self, unicode_filename: str) -> None:
         match = _RE_INVALID_FILENAME.findall(unicode_filename)
         if match:
             raise InvalidCharError(
                 INVALID_CHAR_ERR_MSG_TMPL.format(
                     invalid=findall_to_str(match), value=repr(unicode_filename)
-                )
+                ),
+                platform=Platform.UNIVERSAL,
             )
 
     def __validate_win_filename(self, unicode_filename: str) -> None:
@@ -260,6 +279,7 @@ def validate_filename(
     max_len: int = _DEFAULT_MAX_FILENAME_LEN,
     fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
+    additional_reserved_names: Optional[Sequence[str]] = None,
 ) -> None:
     """Verifying whether the ``filename`` is a valid file name or not.
 
@@ -283,10 +303,13 @@ def validate_filename(
 
             Defaults to ``255``.
         fs_encoding:
-            Filesystem encoding that used to calculate the byte length of the filename.
-            If |None|, get the value from the execution environment.
+            Filesystem encoding that is used to calculate the byte length of the filename.
+            If |None|, get the encoding from the execution environment.
         check_reserved:
-            If |True|, check reserved names of the ``platform``.
+            If |True|, check the reserved names of the ``platform``.
+        additional_reserved_names:
+            Additional reserved names to check.
+            Case insensitive.
 
     Raises:
         ValidationError (ErrorReason.INVALID_LENGTH):
@@ -297,7 +320,7 @@ def validate_filename(
             The following characters are also invalid for Windows platforms:
             |invalid_win_filename_chars|.
         ValidationError (ErrorReason.RESERVED_NAME):
-            If the ``filename`` equals reserved name by OS.
+            If the ``filename`` equals the reserved name by OS.
             Windows reserved name is as follows:
             ``"CON"``, ``"PRN"``, ``"AUX"``, ``"NUL"``, ``"COM[1-9]"``, ``"LPT[1-9]"``.
 
@@ -315,6 +338,7 @@ def validate_filename(
         max_len=max_len,
         fs_encoding=fs_encoding,
         check_reserved=check_reserved,
+        additional_reserved_names=additional_reserved_names,
     ).validate(filename)
 
 
@@ -325,12 +349,15 @@ def is_valid_filename(
     max_len: Optional[int] = None,
     fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
+    additional_reserved_names: Optional[Sequence[str]] = None,
 ) -> bool:
     """Check whether the ``filename`` is a valid name or not.
 
     Args:
         filename:
             A filename to be checked.
+        platform:
+            Target platform name of the filename.
 
     Example:
         :ref:`example-is-valid-filename`
@@ -345,6 +372,7 @@ def is_valid_filename(
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
         check_reserved=check_reserved,
+        additional_reserved_names=additional_reserved_names,
     ).is_valid(filename)
 
 
@@ -354,8 +382,10 @@ def sanitize_filename(
     platform: Optional[PlatformType] = None,
     max_len: Optional[int] = _DEFAULT_MAX_FILENAME_LEN,
     fs_encoding: Optional[str] = None,
-    check_reserved: bool = True,
-    null_value_handler: Optional[NullValueHandler] = None,
+    check_reserved: Optional[bool] = None,
+    null_value_handler: Optional[ValidationErrorHandler] = None,
+    reserved_name_handler: Optional[ValidationErrorHandler] = None,
+    additional_reserved_names: Optional[Sequence[str]] = None,
     validate_after_sanitize: bool = False,
 ) -> PathType:
     """Make a valid filename from a string.
@@ -369,9 +399,8 @@ def sanitize_filename(
             - |invalid_filename_chars|
             - for Windows (or universal) only: |invalid_win_filename_chars|
 
-        - Append underscore (``"_"``) at the tail of the return value if a sanitized name
-          is one of the reserved names by operating systems
-          (only when ``check_reserved`` is |True|).
+        - Replace a value if a sanitized value is a reserved name by operating systems
+          with a specified handler by ``reserved_name_handler``.
 
     Args:
         filename: Filename to sanitize.
@@ -386,14 +415,32 @@ def sanitize_filename(
             Truncate the name length if the ``filename`` length exceeds this value.
             Defaults to ``255``.
         fs_encoding:
-            Filesystem encoding that used to calculate the byte length of the filename.
-            If |None|, get the value from the execution environment.
+            Filesystem encoding that is used to calculate the byte length of the filename.
+            If |None|, get the encoding from the execution environment.
         check_reserved:
-            If |True|, sanitize reserved names of the ``platform``.
+            [Deprecated] Use 'reserved_name_handler' instead.
         null_value_handler:
             Function called when a value after sanitization is an empty string.
-            Defaults to ``pathvalidate.handler.return_null_string()`` function that
-            return an empty string.
+            You can specify predefined handlers:
+
+                - :py:func:`~.handler.NullValueHandler.return_null_string`
+                - :py:func:`~.handler.NullValueHandler.return_timestamp`
+                - :py:func:`~.handler.raise_error`
+
+            Defaults to :py:func:`.handler.NullValueHandler.return_null_string` that just return ``""``.
+        reserved_name_handler:
+            Function called when a value after sanitization is a reserved name.
+            You can specify predefined handlers:
+
+                - :py:meth:`~.handler.ReservedNameHandler.add_leading_underscore`
+                - :py:meth:`~.handler.ReservedNameHandler.add_trailing_underscore`
+                - :py:meth:`~.handler.ReservedNameHandler.as_is`
+                - :py:func:`~.handler.raise_error`
+
+            Defaults to :py:func:`.handler.add_trailing_underscore`.
+        additional_reserved_names:
+            Additional reserved names to sanitize.
+            Case insensitive.
         validate_after_sanitize:
             Execute validation after sanitization to the file name.
 
@@ -409,11 +456,21 @@ def sanitize_filename(
         :ref:`example-sanitize-filename`
     """
 
+    if check_reserved is not None:
+        warnings.warn(
+            "'check_reserved' is deprecated. Use 'reserved_name_handler' instead.",
+            DeprecationWarning,
+        )
+
+        if check_reserved is False:
+            reserved_name_handler = ReservedNameHandler.as_is
+
     return FileNameSanitizer(
         platform=platform,
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
-        check_reserved=check_reserved,
         null_value_handler=null_value_handler,
+        reserved_name_handler=reserved_name_handler,
+        additional_reserved_names=additional_reserved_names,
         validate_after_sanitize=validate_after_sanitize,
     ).sanitize(filename, replacement_text)
