@@ -6,20 +6,25 @@ import ntpath
 import os.path
 import posixpath
 import re
+import warnings
+from collections.abc import Sequence
 from pathlib import Path, PurePath
-from typing import List, Optional, Pattern, Tuple
+from re import Pattern
+from typing import Final, Optional
 
 from ._base import AbstractSanitizer, AbstractValidator, BaseFile, BaseValidator
-from ._common import findall_to_str, to_str, validate_pathtype
+from ._common import findall_to_str, is_nt_abspath, to_str, validate_pathtype
 from ._const import _NTFS_RESERVED_FILE_NAMES, DEFAULT_MIN_LEN, INVALID_CHAR_ERR_MSG_TMPL, Platform
 from ._filename import FileNameSanitizer, FileNameValidator
 from ._types import PathType, PlatformType
-from .error import ErrorReason, InvalidCharError, ReservedNameError, ValidationError
-from .handler import NullValueHandler
+from .error import ErrorAttrKey, ErrorReason, InvalidCharError, ReservedNameError, ValidationError
+from .handler import ReservedNameHandler, ValidationErrorHandler
 
 
-_RE_INVALID_PATH = re.compile(f"[{re.escape(BaseFile._INVALID_PATH_CHARS):s}]", re.UNICODE)
-_RE_INVALID_WIN_PATH = re.compile(f"[{re.escape(BaseFile._INVALID_WIN_PATH_CHARS):s}]", re.UNICODE)
+_RE_INVALID_PATH: Final = re.compile(f"[{re.escape(BaseFile._INVALID_PATH_CHARS):s}]", re.UNICODE)
+_RE_INVALID_WIN_PATH: Final = re.compile(
+    f"[{re.escape(BaseFile._INVALID_WIN_PATH_CHARS):s}]", re.UNICODE
+)
 
 
 class FilePathSanitizer(AbstractSanitizer):
@@ -28,8 +33,9 @@ class FilePathSanitizer(AbstractSanitizer):
         max_len: int = -1,
         fs_encoding: Optional[str] = None,
         platform: Optional[PlatformType] = None,
-        check_reserved: bool = True,
-        null_value_handler: Optional[NullValueHandler] = None,
+        null_value_handler: Optional[ValidationErrorHandler] = None,
+        reserved_name_handler: Optional[ValidationErrorHandler] = None,
+        additional_reserved_names: Optional[Sequence[str]] = None,
         normalize: bool = True,
         validate_after_sanitize: bool = False,
         validator: Optional[AbstractValidator] = None,
@@ -41,15 +47,17 @@ class FilePathSanitizer(AbstractSanitizer):
                 min_len=DEFAULT_MIN_LEN,
                 max_len=max_len,
                 fs_encoding=fs_encoding,
-                check_reserved=check_reserved,
+                check_reserved=True,
+                additional_reserved_names=additional_reserved_names,
                 platform=platform,
             )
         super().__init__(
             max_len=max_len,
             fs_encoding=fs_encoding,
-            check_reserved=check_reserved,
             validator=fpath_validator,
             null_value_handler=null_value_handler,
+            reserved_name_handler=reserved_name_handler,
+            additional_reserved_names=additional_reserved_names,
             platform=platform,
             validate_after_sanitize=validate_after_sanitize,
         )
@@ -58,8 +66,9 @@ class FilePathSanitizer(AbstractSanitizer):
         self.__fname_sanitizer = FileNameSanitizer(
             max_len=self.max_len,
             fs_encoding=fs_encoding,
-            check_reserved=check_reserved,
             null_value_handler=null_value_handler,
+            reserved_name_handler=reserved_name_handler,
+            additional_reserved_names=additional_reserved_names,
             platform=self.platform,
             validate_after_sanitize=validate_after_sanitize,
         )
@@ -78,18 +87,17 @@ class FilePathSanitizer(AbstractSanitizer):
                 if isinstance(value, PurePath):
                     raise
 
-                return self._null_value_handler(e)
+                return self._null_value_handler(e)  # type: ignore
             raise
 
         unicode_filepath = to_str(value)
-
         drive, unicode_filepath = self.__split_drive(unicode_filepath)
         unicode_filepath = self._sanitize_regexp.sub(replacement_text, unicode_filepath)
         if self.__normalize and unicode_filepath:
             unicode_filepath = os.path.normpath(unicode_filepath)
         sanitized_path = unicode_filepath
 
-        sanitized_entries: List[str] = []
+        sanitized_entries: list[str] = []
         if drive:
             sanitized_entries.append(drive)
         for entry in sanitized_path.replace("\\", "/").split("/"):
@@ -118,9 +126,9 @@ class FilePathSanitizer(AbstractSanitizer):
             self._validator.validate(sanitized_path)
 
         if isinstance(value, PurePath):
-            return Path(sanitized_path)
+            return Path(sanitized_path)  # type: ignore
 
-        return sanitized_path
+        return sanitized_path  # type: ignore
 
     def _get_sanitize_regexp(self) -> Pattern[str]:
         if self._is_windows(include_universal=True):
@@ -136,14 +144,14 @@ class FilePathSanitizer(AbstractSanitizer):
 
 
 class FilePathValidator(BaseValidator):
-    _RE_NTFS_RESERVED = re.compile(
+    _RE_NTFS_RESERVED: Final = re.compile(
         "|".join(f"^/{re.escape(pattern)}$" for pattern in _NTFS_RESERVED_FILE_NAMES),
         re.IGNORECASE,
     )
-    _MACOS_RESERVED_FILE_PATHS = ("/", ":")
+    _MACOS_RESERVED_FILE_PATHS: Final = ("/", ":")
 
     @property
-    def reserved_keywords(self) -> Tuple[str, ...]:
+    def reserved_keywords(self) -> tuple[str, ...]:
         common_keywords = super().reserved_keywords
 
         if any([self._is_universal(), self._is_posix(), self._is_macos()]):
@@ -161,17 +169,24 @@ class FilePathValidator(BaseValidator):
         fs_encoding: Optional[str] = None,
         platform: Optional[PlatformType] = None,
         check_reserved: bool = True,
+        additional_reserved_names: Optional[Sequence[str]] = None,
     ) -> None:
         super().__init__(
             min_len=min_len,
             max_len=max_len,
             fs_encoding=fs_encoding,
             check_reserved=check_reserved,
+            additional_reserved_names=additional_reserved_names,
             platform=platform,
         )
 
         self.__fname_validator = FileNameValidator(
-            min_len=min_len, max_len=max_len, check_reserved=check_reserved, platform=platform
+            min_len=min_len,
+            max_len=self.max_len,
+            fs_encoding=fs_encoding,
+            check_reserved=check_reserved,
+            additional_reserved_names=additional_reserved_names,
+            platform=platform,
         )
 
         if self._is_windows(include_universal=True):
@@ -190,9 +205,11 @@ class FilePathValidator(BaseValidator):
         unicode_filepath = to_str(tail)
         byte_ct = len(unicode_filepath.encode(self._fs_encoding))
         err_kwargs = {
-            "reason": ErrorReason.INVALID_LENGTH,
-            "platform": self.platform,
-            "fs_encoding": self._fs_encoding,
+            ErrorAttrKey.REASON: ErrorReason.INVALID_LENGTH,
+            ErrorAttrKey.PLATFORM: self.platform,
+            ErrorAttrKey.FS_ENCODING: self._fs_encoding,
+            ErrorAttrKey.BYTE_COUNT: byte_ct,
+            ErrorAttrKey.VALUE: unicode_filepath,
         }
 
         if byte_ct > self.max_len:
@@ -218,7 +235,7 @@ class FilePathValidator(BaseValidator):
             if not entry or entry in (".", ".."):
                 continue
 
-            self.__fname_validator._validate_reserved_keywords(entry)
+            self.__fname_validator.validate(entry)
 
         if self._is_windows(include_universal=True):
             self.__validate_win_filepath(unicode_filepath)
@@ -227,7 +244,18 @@ class FilePathValidator(BaseValidator):
 
     def validate_abspath(self, value: PathType) -> None:
         is_posix_abs = posixpath.isabs(value)
-        is_nt_abs = ntpath.isabs(value)
+        is_nt_abs = is_nt_abspath(to_str(value))
+
+        if any([self._is_windows() and is_nt_abs, self._is_posix() and is_posix_abs]):
+            return
+
+        if self._is_universal() and any([is_nt_abs, is_posix_abs]):
+            ValidationError(
+                "platform-independent absolute file path is not supported",
+                platform=self.platform,
+                reason=ErrorReason.MALFORMED_ABS_PATH,
+            )
+
         err_object = ValidationError(
             description=(
                 "an invalid absolute file path ({}) for the platform ({}).".format(
@@ -240,25 +268,13 @@ class FilePathValidator(BaseValidator):
             reason=ErrorReason.MALFORMED_ABS_PATH,
         )
 
-        if any([self._is_windows() and is_nt_abs, self._is_linux() and is_posix_abs]):
-            return
-
-        if self._is_universal() and any([is_posix_abs, is_nt_abs]):
-            ValidationError(
-                description=(
-                    ("POSIX style" if is_posix_abs else "NT style")
-                    + " absolute file path found. expected a platform-independent file path."
-                ),
-                platform=self.platform,
-                reason=ErrorReason.MALFORMED_ABS_PATH,
-            )
-
         if self._is_windows(include_universal=True) and is_posix_abs:
             raise err_object
 
-        drive, _tail = ntpath.splitdrive(value)
-        if not self._is_windows() and drive and is_nt_abs:
-            raise err_object
+        if not self._is_windows():
+            drive, _tail = ntpath.splitdrive(value)
+            if drive and is_nt_abs:
+                raise err_object
 
     def __validate_unix_filepath(self, unicode_filepath: str) -> None:
         match = _RE_INVALID_PATH.findall(unicode_filepath)
@@ -299,6 +315,7 @@ def validate_filepath(
     max_len: Optional[int] = None,
     fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
+    additional_reserved_names: Optional[Sequence[str]] = None,
 ) -> None:
     """Verifying whether the ``file_path`` is a valid file path or not.
 
@@ -321,10 +338,13 @@ def validate_filepath(
                 - ``Windows``: 260
                 - ``universal``: 260
         fs_encoding (Optional[str], optional):
-            Filesystem encoding that used to calculate the byte length of the file path.
-            If |None|, get the value from the execution environment.
+            Filesystem encoding that is used to calculate the byte length of the file path.
+            If |None|, get the encoding from the execution environment.
         check_reserved (bool, optional):
-            If |True|, check reserved names of the ``platform``.
+            If |True|, check the reserved names of the ``platform``.
+            Defaults to |True|.
+        additional_reserved_names (Optional[Sequence[str]], optional):
+            Additional reserved names to check.
 
     Raises:
         ValidationError (ErrorReason.INVALID_CHARACTER):
@@ -335,7 +355,7 @@ def validate_filepath(
         ValidationError (ErrorReason.INVALID_LENGTH):
             If the ``file_path`` is longer than ``max_len`` characters.
         ValidationError:
-            If ``file_path`` include invalid values.
+            If ``file_path`` includes invalid values.
 
     Example:
         :ref:`example-validate-file-path`
@@ -351,6 +371,7 @@ def validate_filepath(
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
         check_reserved=check_reserved,
+        additional_reserved_names=additional_reserved_names,
     ).validate(file_path)
 
 
@@ -361,12 +382,15 @@ def is_valid_filepath(
     max_len: Optional[int] = None,
     fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
+    additional_reserved_names: Optional[Sequence[str]] = None,
 ) -> bool:
     """Check whether the ``file_path`` is a valid name or not.
 
     Args:
         file_path:
             A filepath to be checked.
+        platform:
+            Target platform name of the file path.
 
     Example:
         :ref:`example-is-valid-filepath`
@@ -381,6 +405,7 @@ def is_valid_filepath(
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
         check_reserved=check_reserved,
+        additional_reserved_names=additional_reserved_names,
     ).is_valid(file_path)
 
 
@@ -390,8 +415,10 @@ def sanitize_filepath(
     platform: Optional[PlatformType] = None,
     max_len: Optional[int] = None,
     fs_encoding: Optional[str] = None,
-    check_reserved: bool = True,
-    null_value_handler: Optional[NullValueHandler] = None,
+    check_reserved: Optional[bool] = None,
+    null_value_handler: Optional[ValidationErrorHandler] = None,
+    reserved_name_handler: Optional[ValidationErrorHandler] = None,
+    additional_reserved_names: Optional[Sequence[str]] = None,
     normalize: bool = True,
     validate_after_sanitize: bool = False,
 ) -> PathType:
@@ -399,16 +426,15 @@ def sanitize_filepath(
 
     To make a valid file path, the function does the following:
 
-        - replace invalid characters for a file path within the ``file_path``
+        - Replace invalid characters for a file path within the ``file_path``
           with the ``replacement_text``. Invalid characters are as follows:
 
             - unprintable characters
             - |invalid_file_path_chars|
             - for Windows (or universal) only: |invalid_win_file_path_chars|
 
-        - Append underscore (``"_"``) at the tail of the name if sanitized name
-          is one of the reserved names by operating systems
-          (only when ``check_reserved`` is |True|).
+        - Replace a value if a sanitized value is a reserved name by operating systems
+          with a specified handler by ``reserved_name_handler``.
 
     Args:
         file_path:
@@ -430,13 +456,32 @@ def sanitize_filepath(
                 - ``Windows``: 260
                 - ``universal``: 260
         fs_encoding:
-            Filesystem encoding that used to calculate the byte length of the file path.
-            If |None|, get the value from the execution environment.
+            Filesystem encoding that is used to calculate the byte length of the file path.
+            If |None|, get the encoding from the execution environment.
         check_reserved:
-            If |True|, sanitize reserved names of the ``platform``.
+            [Deprecated] Use 'reserved_name_handler' instead.
         null_value_handler:
             Function called when a value after sanitization is an empty string.
-            Defaults to ``pathvalidate.handler.return_null_string()`` that just return ``""``.
+            You can specify predefined handlers:
+
+                - :py:func:`.handler.NullValueHandler.return_null_string`
+                - :py:func:`.handler.NullValueHandler.return_timestamp`
+                - :py:func:`.handler.raise_error`
+
+            Defaults to :py:func:`.handler.NullValueHandler.return_null_string` that just return ``""``.
+        reserved_name_handler:
+            Function called when a value after sanitization is one of the reserved names.
+            You can specify predefined handlers:
+
+                - :py:meth:`~.handler.ReservedNameHandler.add_leading_underscore`
+                - :py:meth:`~.handler.ReservedNameHandler.add_trailing_underscore`
+                - :py:meth:`~.handler.ReservedNameHandler.as_is`
+                - :py:func:`~.handler.raise_error`
+
+            Defaults to :py:func:`.handler.add_trailing_underscore`.
+        additional_reserved_names:
+            Additional reserved names to sanitize.
+            Case insensitive.
         normalize:
             If |True|, normalize the the file path.
         validate_after_sanitize:
@@ -454,12 +499,22 @@ def sanitize_filepath(
         :ref:`example-sanitize-file-path`
     """
 
+    if check_reserved is not None:
+        warnings.warn(
+            "'check_reserved' is deprecated. Use 'reserved_name_handler' instead.",
+            DeprecationWarning,
+        )
+
+        if check_reserved is False:
+            reserved_name_handler = ReservedNameHandler.as_is
+
     return FilePathSanitizer(
         platform=platform,
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
-        check_reserved=check_reserved,
         normalize=normalize,
         null_value_handler=null_value_handler,
+        reserved_name_handler=reserved_name_handler,
+        additional_reserved_names=additional_reserved_names,
         validate_after_sanitize=validate_after_sanitize,
     ).sanitize(file_path, replacement_text)
